#!/usr/local/bin/perl
#
$usg = 'convert.pl: Convert configuration file written in "sh" syntax
            into "tcl" and "perl" syntax.  All WWFS-related programs
            read one of these configuration files.
Usage: convert.pl <config-files...>
';
# Author: Youki Kadobayashi <youki@wide.ad.jp>
# This file is part of WWFS.
#

if ($#ARGV < $[) {
    die $usg;
}

foreach $i (@ARGV) {
    # Step 1.
    # Convert "sh" style configuration file into "tcl" style.
    open(IN, $i) || warn "$i: $!\n", next;
    $o = &suffix($i, "tcl");
    open(OUT, "> $o") || warn "$o: $!\n", next;
    while (<IN>) {
	/=/ && do {
	    s/(\w+)=(.*)$/set \1 "\2"/;
	    print OUT;
	}
    }
    close(OUT);
    close(IN);

    # Step 2.
    # Convert "sh" style configuration file into "perl" style.
    open(IN, $i) || warn "$i: $!\n", next;
    $o = &suffix($i, "pl");
    open(OUT, "> $o") || warn "$o: $!\n", next;
    while (<IN>) {
	/=/ && do {
	    s/(\w+)=(.*)$/\$\1="\2";/;
	    print OUT;
	};
    }
    close(OUT);
    close(IN);
}

sub suffix {
    local ($name, $suffix) = @_;
    # replace suffix.

    $name =~ s/\.\w+$/./;
    return $name . $suffix;
}

# Local Variables:
# mode: perl
# End:
