/* callout; borrowed and modified amd5.3alpha14 source */
static char *AtFSid = "$Header: callout.c[109.0] Wed Nov 24 03:47:03 1993 youki-k@is.aist-nara.ac.jp saved $";
/*
 * Copyright (c) 1989 Jan-Simon Pendry
 * Copyright (c) 1989 Imperial College of Science, Technology & Medicine
 * Copyright (c) 1989 The Regents of the University of California.
 * All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Jan-Simon Pendry at Imperial College, London.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	%W% (Berkeley) %G%
 *
 * $Id: clock.c,v 5.2.2.1 1992/02/09 15:08:20 jsp beta $
 *
 */

#include "wfs.h"
#include "util.h"
#include "global.h"

/*
 * Callouts.
 *
 * Modelled on kernel object of the same name.
 * See usual references.
 *
 * Use of a heap-based mechanism was rejected:
 * 1.  more complex implementation needed.
 * 2.  not obvious that a list is too slow for Amd.
 */

struct callout {
	qelem	q;			/* list of callouts */
	void	(*c_fn)();		/* function to call */
	void	*c_closure;		/* closure to pass to call */
	time_t	c_time;			/* time of call */
};

extern qelem callouts;
qelem callouts = { &callouts, &callouts };
time_t next_softclock;			/* Time of next call to softclock() */

/* Schedule a callout */
void *
timeout_set(t, fn, closure)
time_t t;
void (*fn)();
void *closure;
{
	callout *cp;

	/*
	 * Allocate and fill in a new callout structure
	 */
	callout *cpnew = ALLOC(struct callout);
	cpnew->c_closure = closure;
	cpnew->c_fn = fn;
	cpnew->c_time = t;

	if (t < next_softclock)
		next_softclock = t;

	/*
	 * Find the correct place in the list
	 */
	ITER(cp, callout, &callouts) {
		if (cp->c_time >= t)
			break;
	}
	q_insert(cpnew, PREV(callout, cp));

	/*
	 * Return callout
	 */
	return (void *) cpnew;
}

/* Reconfigure a callout */
void
timeout_reset(t, cp)
time_t t;
callout *cp;
{
	callout *p;

	if (cp->c_time < t) {
		if (NEXT(callout, &callouts) == cp) {
			next_softclock = MIN(t, NEXT(callout, cp)->c_time);
		}
		q_remove(cp);
		ITER(p, callout, &callouts) {
			if (p->c_time > t)
				break;
		}
		q_insert(cp, PREV(callout, p));
	} else if (t < cp->c_time) {
		q_remove(cp);
		REVITER(p, callout, &callouts) {
			if (p->c_time < t)
				break;
		}
		q_insert(cp, p);
		if (p == (callout *) &callouts) {
			next_softclock = t;
		}
	} else {
		syslog(LOG_WARNING, "unneeded timeout_reset");
	}
}

/* De-schedule a callout */
void
timeout_clear(cp)
callout *cp;
{
	q_remove(cp);
	FREE(cp);
	return;
}

/* Reschedule after clock changed */
void
timeout_reschedule(now, then)
time_t now;
time_t then;
{
	callout *cp;

	ITER(cp, callout, &callouts) {
		if (cp->c_time >= now && cp->c_time <= then) {
#ifdef DEBUG
			dlog("rescheduling job %ld back %d seconds",
			     cp->c_time, cp->c_time - now);
#endif
			next_softclock = cp->c_time = now;
		}
	}
}

/* Clock handler */
int
softclock()
{
	callout *cp;

	realtime();

	/* While there are more callouts waiting... */
	while ((cp = NEXT(callout, &callouts))
	       != HEAD(callout, &callouts)
	       && cp->c_time <= now) {
		/*
		 * Extract first from list, save fn & closure and
		 * unlink callout from list and free.  Finally call
		 * function.
		 * The free is done first because it is quite common
		 * that the function will call timeout() and try to
		 * allocate a callout
		 */
		void (*fn)() = cp->c_fn;
		void *closure = cp->c_closure;
		
		q_remove(cp);
		FREE(cp);
#ifdef DEBUG
		/*dlog("Calling %#x(%#x)", fn, closure);*/
#endif /* DEBUG */
		(*fn)(closure);
	}

	/*
	 * Return number of seconds to next event, or 0 if there is no
	 * event.
	 */
	if (MORE(&callouts)) {
		cp = NEXT(callout, &callouts);
		return cp->c_time - now;
	}
	return 0;
}
