/* 
 * WorldWide File System
 * Copyright (c) 1992,1993 Youki Kadobayashi
 * Copyright (c) 1992,1993 Osaka University
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that the following conditions
 * are met:
 * 1. Both the copyright notice and this permission notice appear in
 *    all copies of the software, derivative works or modified versions,
 *    and any portions thereof, and that both notices appear in
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Osaka University
 *      and its contributors.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPER ``AS IS'' AND OSAKA
 * UNIVERSITY DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
 * WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Osaka University requests users of this software to return to
 *
 *  Youki Kadobayashi
 *  Department of Information and Computer Sciences
 *  Osaka University, Toyonaka 560, Osaka, Japan
 *
 * any improvements or extensions that they make and grant Osaka
 * University the rights to redistribute these changes.
 */
/* directory management routines */
static char *AtFSid = "$Header: dir.c[109.0] Wed Nov 24 03:47:07 1993 youki-k@is.aist-nara.ac.jp saved $";

#include <sys/types.h>
#include <sys/stat.h>		/* for struct stat */
#include <fcntl.h>		/* for O_WRONLY etc. */
#include <string.h>
#include <stdio.h>
#include "wfs.h"
#include "util.h"
#include "global.h"

/* not used actually. */
struct save_file {
	long	fileid;
	int	namelen;
	char	*name;
	int	linklen;
	char	*link;
	struct	fattr *fattr;
};

/* not used actually. */
struct save_dir {
#define	WF_DIR_MAGIC	0xbaddeaL
	long	magic;
	int	namelen;
	char	*name;		/* entire directory name */
	long	ctime;
	int	nfile;
	struct	save_file *file;
};

wf_dir *dir_load();

typedef struct {
	qelem	q;
	wf_dir	*dir;
} wf_dirq;

extern qelem dir_head;
qelem dir_head = { &dir_head, &dir_head };		/* chain of wf_dirq */
static void *dir_callout;

extern qelem olddir_head;
qelem olddir_head = { &olddir_head, &olddir_head };	/* chain of wf_dir */

static void dir_gc(), dir_free();

void
dir_start()
{
	dir_callout = timeout_set(gettime()+60, dir_gc, 0);
}

static void
dir_gc()
{
	wf_dirq *p, *p2;

	ITER2(p, p2, wf_dirq, &dir_head) {
		if (p->dir->flag & WF_DIR_WIREDDOWN || p->dir->n_xfer)
			continue;
		p->dir->idle += 60;
		if (p->dir->idle > WF_DIR_TTLMEM) {
			q_remove(p);
			dir_free(p->dir);
			FREE(p);
		}
	}
	dir_callout = timeout_set(gettime()+60, dir_gc, 0);
}

void
dir_addfile(dirp, filep)
wf_dir *dirp;
wf_file *filep;
{
	q_insert(filep, LAST(wf_file, &dirp->file->q));
}

void
dir_deletefile(filep)
wf_file *filep;
{
	q_remove(filep);
}

/* subdirectory */
wf_file *
dir_adddir(dirp, name, volid, dirid)
wf_dir *dirp;
char *name;
long volid, dirid;
{
	wf_file *filep;

	filep = file_alloc();
	attr_initdir(&filep->fattr);
	filep->name = strdup(name);
	filep->fattr.fsid = volid;
	filep->id = filep->fattr.fileid = dirid;
	filep->crc = updcrc(0L, name, strlen(name)+1);
	dir_addfile(dirp, filep);
	return filep;
}

static wf_dir *
dir_alloc(volp)
wf_vol *volp;
{
	wf_dir *dirp;
	wf_dirq *q;

	dirp = ALLOC(wf_dir);
	dirp->file = (wf_file *) q_alloc();
	q_insert(dirp, volp->dirp);

	q = ALLOC(wf_dirq);
	q->dir = dirp;
	q_insert(q, &dir_head);
	return dirp;
}

static void
dir_free(dirp)
wf_dir *dirp;
{
	wf_file *filep, *filep2;

	file_uncache(dirp);
	if (dirp->id == 0) {
		syslog(LOG_WARNING, "dirp->id == 0");
	}
#ifdef DEBUG_GC
	dlog("free wf_dir \"%s\"", dirp->name);
#endif
	ITER2 (filep, filep2, wf_file, &dirp->file->q) {
		file_free(filep);
	}
	FREE(dirp->file);
	q_remove(dirp);
	FREE(dirp);
}

static void
dir_makeobsolete(dirp)
wf_dir *dirp;
{
	q_remove(dirp);
	q_insert(dirp, &olddir_head);
}

wf_dir *
dir_new(volp, id, dir, subdir)
wf_vol *volp;
long id;
char *dir, *subdir;
{
	wf_dir *dirp;
	int dirlen;

	if ((dirp = dir_findcache(volp, id)) != (wf_dir *)0) {
		dir_makeobsolete(dirp);
	}
	dirp = dir_alloc(volp);
	dirp->id = id;
	dirp->ctime = gettime();
	if (dir && subdir) {
		dirlen = strlen(dir);
		dirp->name = malloc(dirlen + strlen(subdir) + 2);
		if (dirlen && dir[dirlen-1] == '/') {
			sprintf(dirp->name, "%s%s", dir, subdir);
		} else if (! strcmp(subdir, ".")) {
			/* avoid "foo/./././." */
			sprintf(dirp->name, "%s", dir);
		} else {
			sprintf(dirp->name, "%s/%s", dir, subdir);
		}
	} else {
		dirp->name = strdup("/");
	}
	return dirp;
}

wf_dir *
dir_findcache(volp, dirid)
wf_vol *volp;
long dirid;
{
	wf_dir *dirp;
	qelem *q;

	q = &volp->dirp->q;
	/* search memory cache */
	ITER(dirp, wf_dir, q) {
		if (dirp->id == dirid) {
			if (dirp->idle > 0)
				dirp->idle = 0;
			goto found;
		}
	}

	/* search disk cache */
	dirp = dir_load(volp, dirid);

 found:
	/* check freshness */
	if (dirp && !(dirp->flag & WF_DIR_WIREDDOWN)) {
		if (gettime() - dirp->ctime > WF_DIR_TTLDISK
		    || volp->mtime > dirp->ctime)
			dirp->flag |= WF_DIR_OBSOLETE;
	}
	return dirp;
}

wf_dir *
dir_load(volp, dirid)
wf_vol *volp;
long dirid;
{
	wf_dir *dirp;
	FILE *fp;
	long magic;
	int namelen, qlen;
	wf_file *filep;
	struct stat statbuf;
	char path[MAXPATHLEN];

	sprintf(path, "%s/cache/%lx/dir/%lx", cs_topdir, volp->id, dirid);
	stat(path, &statbuf);
	if (statbuf.st_size == 0) return NULL;
	fp = fopen(path, "r");
	if (fp == NULL) return NULL;

	/* allocate dirp */
	dirp = dir_alloc(volp);
	dirp->id = dirid;

	/* read per-directory info */
	WF_READ(fp, magic);
	if (magic != WF_DIR_MAGIC) {
		syslog(LOG_WARNING, "dir magic mismatch");
		fclose(fp);
		return NULL;
	}
	WF_READ(fp, namelen);
	dirp->name = malloc(namelen);
	fread(dirp->name, namelen, 1, fp);
	WF_READ(fp, dirp->ctime);

	/* read per-file info */
	WF_READ(fp, qlen);
	while (qlen--) {
		filep = file_alloc();
		dir_addfile(dirp, filep);
		WF_READ(fp, filep->id);
		filep->fattr.fileid = filep->id;
		WF_READ(fp, namelen);
		filep->name = malloc(namelen);
		fread(filep->name, namelen, 1, fp);
		WF_READ(fp, namelen);
		if (namelen) {
			filep->link = malloc(namelen);
			fread(filep->link, namelen, 1, fp);
		}

		WF_READ(fp, filep->fattr.type);
		WF_READ(fp, filep->fattr.mode);
		WF_READ(fp, filep->fattr.nlink);
		WF_READ(fp, filep->fattr.uid);
		WF_READ(fp, filep->fattr.gid);
		WF_READ(fp, filep->fattr.size);
		WF_READ(fp, filep->fattr.blocksize);
		WF_READ(fp, filep->fattr.rdev);
		WF_READ(fp, filep->fattr.blocks);
		WF_READ(fp, filep->fattr.fsid);
		WF_READ(fp, filep->crc);
		WF_READ(fp, filep->fattr.atime.seconds);
		WF_READ(fp, filep->fattr.atime.useconds);
		WF_READ(fp, filep->fattr.mtime.seconds);
		WF_READ(fp, filep->fattr.mtime.useconds);
		WF_READ(fp, filep->fattr.ctime.seconds);
		WF_READ(fp, filep->fattr.ctime.useconds);
	}

	fclose(fp);
	return dirp;
}

void
dir_save(volp, dirp, olddirp)
wf_vol *volp;
wf_dir *dirp, *olddirp;
{
	FILE *fp;
	long magic = WF_DIR_MAGIC;
	int namelen, qlen;
	wf_file *filep;
	char path[MAXPATHLEN];

	sprintf(path, "%s/cache/%lx/dir/%lx", cs_topdir, volp->id, dirp->id);
	mkdirs(path);
	fp = fopen(path, "w+");

	/* write per-directory info */
	WF_WRITE(fp, magic);
	namelen = strlen(dirp->name)+1;
	WF_WRITE(fp, namelen);
	fwrite(dirp->name, namelen, 1, fp);
	WF_WRITE(fp, dirp->ctime);

	/* write per-file info */
	qlen = q_len(dirp->file);
	WF_WRITE(fp, qlen);
	ITER(filep, wf_file, &dirp->file->q) {
		WF_WRITE(fp, filep->id);
		namelen = strlen(filep->name)+1;
		WF_WRITE(fp, namelen);
		fwrite(filep->name, namelen, 1, fp);
		if (filep->link) {
			namelen = strlen(filep->link)+1;
			WF_WRITE(fp, namelen);
			fwrite(filep->link, namelen, 1, fp);
		} else {
			namelen = 0;
			WF_WRITE(fp, namelen);
		}

		WF_WRITE(fp, filep->fattr.type);
		WF_WRITE(fp, filep->fattr.mode);
		WF_WRITE(fp, filep->fattr.nlink);
		WF_WRITE(fp, filep->fattr.uid);
		WF_WRITE(fp, filep->fattr.gid);
		WF_WRITE(fp, filep->fattr.size);
		WF_WRITE(fp, filep->fattr.blocksize);
		WF_WRITE(fp, filep->fattr.rdev);
		WF_WRITE(fp, filep->fattr.blocks);
		WF_WRITE(fp, filep->fattr.fsid);
		WF_WRITE(fp, filep->crc);
		WF_WRITE(fp, filep->fattr.atime.seconds);
		WF_WRITE(fp, filep->fattr.atime.useconds);
		WF_WRITE(fp, filep->fattr.mtime.seconds);
		WF_WRITE(fp, filep->fattr.mtime.useconds);
		WF_WRITE(fp, filep->fattr.ctime.seconds);
		WF_WRITE(fp, filep->fattr.ctime.useconds);
	}

	fclose(fp);
	if (olddirp) {
		/* reclaim */
		ITER(filep, wf_file, &olddirp->file->q) {
			if (filep->fattr.type != NFDIR) continue;
			if (! strcmp(filep->name, "..")) continue;
			if (file_findid(dirp, FSID(filep), FID(filep)) == 0) {
				/* directory was removed at fileserver.
				 * reflect the change.
				 */
				dir_dispose(volp, FID(filep));
			}
		}
	}
}

void
dir_realdir(path, volp, dirp)
char *path;
wf_vol *volp;
wf_dir *dirp;
{
	char *p;
	char path2[MAXPATHLEN];
	int fd;

	strcpy(path2, path);
	p = strrchr(path2, '/');
	strcpy(p+1, ".realdir");
	fd = open(path2, O_WRONLY|O_CREAT|O_TRUNC, 0660);
	if (fd < 0)
		dlog("dir_realdir: couldn't create file");
	sprintf(path2, "%s/%s%s", volp->dir, volp->name,
		dirp ? dirp->name : " (directory)");
	write(fd, path2, strlen(path2)+1);
	close(fd);
}

void
dir_dispose(volp, dirid)
wf_vol *volp;
long dirid;
{
	char path[MAXPATHLEN], path2[MAXPATHLEN];

	sprintf(path, "%s/cache/%lx/dir/%lx",
		cs_topdir, volp->id, dirid);
	sprintf(path2, "%s/stale/%s.%d/%lx/dir/%lx",
		cs_topdir, cs_date, cs_serial, volp->id, dirid);
	mkdirs(path2);
	rename(path, path2);
	sprintf(path, "/bin/mv -f %s/cache/%lx/%lx %s/stale/%s.%d/%lx",
		cs_topdir, volp->id, dirid,
		cs_topdir, cs_date, cs_serial, volp->id);
	unix_command(path);
#if 0
	cs_serial++;
#endif
}

void
dir_inspect(reqp, name)
wf_req *reqp;
char *name;
{
	int ret;
	wf_ih ih;
	int id = reqp->id;

	ret = root_name2ih(name, &ih);
	if (ret != WF_REP_OK) {
		req_send(reqp, "513.%03d root_name2ih: error code %d.\n",
			 id, ret);
		return;
	}
	req_send(reqp, "220-%03d dir %s\n", id, name);
	req_send(reqp, "213-%03d name = %s\n", id, PROT(ih.dirp->name));
	req_send(reqp, "213-%03d id = %lx\n", id, ih.dirp->id);
	req_send(reqp, "213-%03d flag = %x\n", id, ih.dirp->flag);
	req_send(reqp, "213-%03d idle = %d\n", id, ih.dirp->idle);
	req_send(reqp, "213-%03d ctime = %ld (%20.20s)\n",
		 id, ih.dirp->ctime, ctime(&ih.dirp->ctime)+4);
	req_send(reqp, "213.%03d n_xfer = %d\n", id, ih.dirp->n_xfer);
}

