/* 
 * WorldWide File System
 * Copyright (c) 1992,1993 Youki Kadobayashi
 * Copyright (c) 1992,1993 Osaka University
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that the following conditions
 * are met:
 * 1. Both the copyright notice and this permission notice appear in
 *    all copies of the software, derivative works or modified versions,
 *    and any portions thereof, and that both notices appear in
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Osaka University
 *      and its contributors.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPER ``AS IS'' AND OSAKA
 * UNIVERSITY DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
 * WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Osaka University requests users of this software to return to
 *
 *  Youki Kadobayashi
 *  Department of Information and Computer Sciences
 *  Osaka University, Toyonaka 560, Osaka, Japan
 *
 * any improvements or extensions that they make and grant Osaka
 * University the rights to redistribute these changes.
 */
/* Read configuration files */
static char *AtFSid = "$Header: parse.c[109.0] Wed Nov 24 03:47:14 1993 youki-k@is.aist-nara.ac.jp saved $";

#include <ctype.h>
#include <stdio.h>
#include <string.h>		/* for strchr */
#include "parse.h"

/* Debugging malloc library of Conor P. Cahill */
#ifdef DEBUG_MALLOC
#include "/usr/local/include/dbmalloc/malloc.h"
#endif

/*
 * Loosely based on STIF (structured text interchange format), which is
 * currently an Internet Draft.
 * We will eventually adopt STIF when RFC comes out.
 */
int
parse_conf(conf, bind_func)
char *conf;
int (*bind_func)();
{
	char line[256];
	int linenum = 0, nerr = 0;
	FILE *fp;
	char *p, *eq;
	char *name, *value;

	fp = fopen(conf, "r");
	if (fp == NULL)
		return -1;

	while (fgets(line, sizeof(line), fp) != NULL) {
		++linenum;
		if ((p = strchr(line, '\n')) != NULL) {
			*p = '\0';
		} else {
			dlog("%s:%d: line too long", conf, linenum);
			++nerr;
		}
		if ((p = strchr(line, '#')) || (p = strchr(line, ';')))
			*p = '\0';
		if (*line == '\0')
			continue;

#define SKIP_SPACE(p, start) \
		for ((p) = (start); *(p) && isspace(*(p)); ++(p))	\
			;
#define TRIM_SPACE(p, start) \
		for ((p) = (start) + strlen(start) - 1;			\
		     (p) >= (start); --(p)) {				\
			if (isspace(*(p)))				\
				*(p) = '\0';				\
		}

		if ((eq = strchr(line, '=')) || (eq = strchr(line, ':'))
		    || (eq = strchr(line, '\t'))) {
			*eq = '\0';
			SKIP_SPACE(p, line);
			name = p;
			TRIM_SPACE(p, name);
			SKIP_SPACE(p, eq+1);
			value = p;
			TRIM_SPACE(p, value);
			if ((*bind_func)(name, value) < 0) {
				dlog("%s:%d: invalid assignment \"%s=%s\"", conf, linenum, name, value);
				++nerr;
			}
		} else {
			dlog("%s:%d: parse error", conf, linenum);
			++nerr;
		}
	}
	fclose(fp);
	if (nerr)
		return -1;
	return 0;
}

int
parse_tailor(tailor, name, value)
wf_tailor *tailor;
char *name;
char *value;
{
	int i, found = 0, ret, len;
	char *p;

	for (i = 0; tailor[i].name; ++i) {
		if (strcasecmp(tailor[i].name, name) == 0) {
			len = strlen(value)+1;
			p = malloc(len);
			bcopy(value, p, len);
			ret = (*tailor[i].assign)(p);
			if (ret < 0) break;	/* invalid parameter */
			found = 1;
			break;
		}
	}
	if (! found) return -1;
	return 0;
}

long
parse_cap(cap, str)		/* comma-separated list of capabilities */
wf_cap *cap;
char *str;
{
	char *p, *p2;
	int i;
	long value = 0;

	p = str;
	do {
		while (isspace(*p)) ++p;
		p2 = strchr(p, ',');
		if (p2) *p2 = '\0';
		for (i = 0; cap[i].name; ++i) {
			if (strcasecmp(cap[i].name, p) == 0) {
				value |= cap[i].value;
				break;
			}
		}
		if (p2) *p2 = ',';
	} while (p2 && (p = p2+1));
	return value;
}

