/* 
 * WorldWide File System
 * Copyright (c) 1992,1993 Youki Kadobayashi
 * Copyright (c) 1992,1993 Osaka University
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that the following conditions
 * are met:
 * 1. Both the copyright notice and this permission notice appear in
 *    all copies of the software, derivative works or modified versions,
 *    and any portions thereof, and that both notices appear in
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Osaka University
 *      and its contributors.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPER ``AS IS'' AND OSAKA
 * UNIVERSITY DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
 * WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Osaka University requests users of this software to return to
 *
 *  Youki Kadobayashi
 *  Department of Information and Computer Sciences
 *  Osaka University, Toyonaka 560, Osaka, Japan
 *
 * any improvements or extensions that they make and grant Osaka
 * University the rights to redistribute these changes.
 */
static char *AtFSid = "$Header: srv.c[109.1] Tue Nov 30 02:47:13 1993 youki-k@is.aist-nara.ac.jp saved $";

#include <ctype.h>
#include <arpa/inet.h>	/* for inet_addr */
#include <netdb.h>	/* for hostent */
extern  int h_errno;	/* for BSDs */
#include "wfs.h"
#include "util.h"
#include "global.h"

#define WF_SRV_PING	(1 * 60)	/* default keepalive interval */

extern qelem srv_head;
qelem srv_head = { &srv_head, &srv_head };

static wf_srv *
srv_alloc()
{
	wf_srv *srvp;

	srvp = ALLOC(wf_srv);
	srvp->connq = (wf_connq *) q_alloc();
	q_insert(srvp, &srv_head);
	return srvp;
}

static
srv_free(srvp)
wf_srv *srvp;
{
	if (srvp->name == NULL || srvp->sin.sin_port == 0) {
		syslog(LOG_WARNING, "attempt to free bogus wf_srv");
	}
	free(srvp->name);
	FREE(srvp->connq);	/* XXX: must free dead connections */
	q_remove(srvp);
	FREE(srvp);
}


static char *err2msg[] = {
	"Authoritative Answer Host not found",
	"Non-Authoritive Host not found, or SERVERFAIL",
	"Non recoverable errors, FORMERR, REFUSED, NOTIMP",
	"Valid name, no data record of requested type",
	"no address, look for MX record",
};

struct hostent *
parse_hostorip(str)
char *str;
{
	static struct hostent h;
	static struct in_addr in;
	struct hostent *hp;
	int retry = 0;

	/*
	 * Brain-damaged corporate networks cannot run DNS.
	 */
	if (isdigit(str[0]) && isdigit(str[strlen(str)-1])) {
		in.s_addr = inet_addr(str);
		h.h_name = str;
		h.h_addrtype = AF_INET;
		h.h_length = sizeof(struct in_addr);
		h.h_addr_list = (char **) calloc(2, sizeof(char *));
		h.h_addr_list[0] = (char *) malloc(sizeof(unsigned long));
		h.h_addr_list[0] = (char *) &in;
		return &h;
	}

	/*
	 * query DNS
	 */
 dns_retry:
	/* occasionally sucks */
	hp = gethostbyname(str);
	if (hp == (struct hostent *)0) {
		dlog("Couldn't resolve \"%s\": %s", str, err2msg[h_errno-1]);
#ifdef WRONG_DNS_CONFIG
		/*
		 * Apparently your DNS has been mis-configured!
		 */
		if (h_errno == TRY_AGAIN) {
			if (++retry < 5) {
				goto dns_retry;
			}
		}
#endif
	}
	return hp;
}

void
srv_resolv(srvp)
wf_srv *srvp;
{
	struct hostent *hp;

	hp = parse_hostorip(srvp->name);
	if (hp == (struct hostent *)0) {
		srv_flag(srvp, WF_SRV_BADNS);
		return;
	}
	srvp->sin.sin_family = hp->h_addrtype;
	bcopy(hp->h_addr_list[0], &srvp->sin.sin_addr, hp->h_length);
}

/* find server by server hostname or IP address */
wf_srv *
srv_findname(name)
char *name;
{
	wf_srv *srvp;

	/*
	 * search memory cache
	 */
	ITER(srvp, wf_srv, &srv_head) {
		if (! strcmp(srvp->name, name)) {
			return srvp;
		}
	}

	/*
	 * not in cache; allocate new server entry
	 */
	srvp = srv_alloc();
	srvp->name = strdup(name);
	srvp->sin.sin_port = htons(21);	/* XXX */
	srvp->protos = WF_PROTO_FTP;	/* XXX */
	srv_resolv(srvp);
	return srvp;
}

wf_srv *
srv_findip(ipaddr)
long ipaddr;	/* network byte order */
{
	wf_srv *srvp;
	ITER(srvp, wf_srv, &srv_head) {
		if (srvp->sin.sin_addr.s_addr == ipaddr) {
			return srvp;
		}
	}
	return (wf_srv *)0;
}

wf_conn *
srv_getconn(srvq)
wf_srvq *srvq;
{
	wf_connq *q;
	qelem *q_head = &srvq->srv->connq->q;

	ITER(q, wf_connq, q_head) {
		if (q->conn->thrdp == (wf_thrd *)0) {
			q->conn->idle = 0;
			q->conn->info = srvq->info;
			return q->conn;
		}
	}
	return (wf_conn *)0;
}

wf_conn *
srv_newconn(srvq)
wf_srvq *srvq;
{
	wf_conn *cp;
	wf_connq *q;

	if (srvq->srv->nconn < WF_SRV_MAXCONN) {
		cp = conn_alloc();
		cp->idle = 0;
		cp->srv = srvq->srv;
		cp->info = srvq->info;
		q = ALLOC(wf_connq);
		q->conn = cp;
		q_insert(q, srvq->srv->connq);
		++srvq->srv->nconn;
		return cp;
	} else {
		return (wf_conn *)0;
	}
}

void
srv_releaseconn(srvp, cp)
wf_srv *srvp;
wf_conn *cp;
{
	wf_connq *q, *q2;
	qelem *q_head;

	q_head = &srvp->connq->q;
	ITER2(q, q2, wf_connq, q_head) {
		if (q->conn == cp) {
			q_remove(q);
			FREE(q);
			warn_negative(--srvp->nconn);
			return;
		}
	}
	dlog("srv_releaseconn: no such connection");
}

static void
srv_reflag(ipaddr)
long ipaddr;
{
	sockaddr_in sin;

	/* It is questionable that service have been restored
	 * in the case of WF_SRV_REFUSED, which probably
	 * indicate interim or long term service shutdown.
	 * But it's worth a retry.
	 */

	/*
	 * clear flag
	 */
	wf_srv *srvp;
	/* Server data structure might be re-allocated; fetch it again. */
	srvp = srv_findip(ipaddr);
	if (! srvp)
		return;
	srvp->touch = gettime();

	if (srvp->flag & WF_SRV_BADNS) {
		/* try to resolv it again */
		srv_resolv(srvp);
	}
	/*
	 * ping
	 */
	srvp->flag = 0;			/* marked again if no improvement */
	bzero((char *)&sin, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = htonl(ipaddr);
	icmp_send(&sin);
}

void
srv_flag(srvp, flag)
wf_srv *srvp;
int flag;
{
	srvp->flag |= flag;
	srvp->touch = gettime();

	timeout_set(gettime() + 12 * 3600, srv_reflag,
		    srvp->sin.sin_addr.s_addr);
	/* hopefully, problem will have been solved after 12 hours... */
}

