/* 
 * WorldWide File System
 * Copyright (c) 1992,1993 Youki Kadobayashi
 * Copyright (c) 1992,1993 Osaka University
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that the following conditions
 * are met:
 * 1. Both the copyright notice and this permission notice appear in
 *    all copies of the software, derivative works or modified versions,
 *    and any portions thereof, and that both notices appear in
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Osaka University
 *      and its contributors.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPER ``AS IS'' AND OSAKA
 * UNIVERSITY DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
 * WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Osaka University requests users of this software to return to
 *
 *  Youki Kadobayashi
 *  Department of Information and Computer Sciences
 *  Osaka University, Toyonaka 560, Osaka, Japan
 *
 * any improvements or extensions that they make and grant Osaka
 * University the rights to redistribute these changes.
 */
/*
 * UIP: WWFS User Interface Protocol.
 * Both UIP/UDP and UIP/TCP are implemented for ease of use from variety
 * of programming languages.
 */
static char *AtFSid = "$Header: uip.c[109.1] Thu Dec 16 20:39:25 1993 youki-k@is.aist-nara.ac.jp saved $";

#include <sys/ioctl.h>		/* for FIONREAD */
#include <ctype.h>
#include <string.h>		/* for strchr */
#include "wfs.h"
#include "global.h"
#include "util.h"

static wf_req *
uip_alloc()
{
	wf_req *p;

	p = ALLOC(wf_req);
	return p;
}

static void
uip_free(p)
wf_req *p;
{
	FREE(p);
}

int
uip_id()
{
	static int id;
	if (++id < 0)
		id = 0;
	return id;
}

static void
uip_emsg(reqp, buf)
wf_req *reqp;
char *buf;
{
	if (strncmp(buf, "progress", 8) == 0) {
	}
	else if (strncmp(buf, "block", 5) == 0) {
	}
	else if (strncmp(buf, "ftptrace", 8) == 0) {
		cs_ftp_debug = reqp;	/* only one for now */
	}
	else {
		req_send(reqp, "520-%03d opt %s\n", reqp->id, buf);
		req_send(reqp, "513.%03d Unknown subcommand.\n", reqp->id);
	}
}

static void
uip_dmsg(reqp, buf)
wf_req *reqp;
char *buf;
{
}

static void
uip_diag(reqp, buf)
wf_req *reqp;
char *buf;
{
#ifdef DEBUG_MALLOC
	static unsigned long hist1, hist2;
#endif

	if (strncmp(buf, "thread", 6) == 0) {
		thrd_inspect(reqp);
	}
	else if (strncmp(buf, "connection", 10) == 0) {
		conn_inspect(reqp);
	}
	else if (strncmp(buf, "file", 4) == 0) {
		file_inspect(reqp, buf + 5);
	}
	else if (strncmp(buf, "dir", 3) == 0) {
		dir_inspect(reqp, buf + 4);
	}
	else if (strncmp(buf, "vol", 3) == 0) {
		vol_inspect(reqp, buf + 4);
	}
	else if (strncmp(buf, "fh", 2) == 0) {
		root_inspect(reqp, buf+3);
	}
#ifdef DEBUG_MALLOC
	else if (strncmp(buf, "mallocdump", 10) == 0) {
		malloc_dump(reqp->so);
	}
	else if (strncmp(buf, "malloclist", 10) == 0) {
		malloc_inuse(&hist2);
		malloc_list(reqp->so, hist1, hist2);
	}
	else if (strncmp(buf, "mallocfrom", 10) == 0) {
		malloc_inuse(&hist1);
	}
#endif
	else {
		req_send(reqp, "520-%03d opt %s\n", reqp->id, buf);
		req_send(reqp, "513.%03d Unknown subcommand.\n", reqp->id);
	}
}

static void
uip_rtoc(reqp, arg)
wf_req *reqp;
char *arg;
{
	int ret;
	wf_ih ih;
	char path[MAXPATHLEN];

	if (strlen(arg) < 5) {
		req_send(reqp, "520-%03d cmd RTOC %s\n", reqp->id, arg);
		req_send(reqp, "513.%03d Invalid argument.\n", reqp->id);
	}
	ret = root_name2ih(arg + 4, &ih);
	if (ret != WF_REP_OK) {
		req_send(reqp, "520-%03d %s\n", reqp->id, arg);
		req_send(reqp, "510.%03d root_name2ih: error code %d.\n",
			 reqp->id, ret);	/* XXX */
		return;
	}
	if (strncmp(arg, "dir", 3) == 0) {
		req_send(reqp, "220.%03d c_p %s/cache/%lx/dir/%lx\n",
			 reqp->id, cs_topdir, ih.child_volp->id, ih.filep->id);
	}
	else if (strncmp(arg, "fil", 3) == 0) {
		file_getpath(path, ih.child_volp, ih.dirp, ih.filep);
		req_send(reqp, "220.%03d c_p %s\n", reqp->id, path);
	}
	else {
		req_send(reqp, "520-%03d opt %s\n", reqp->id, arg);
		req_send(reqp, "513.%03d Unknown subcommand.\n", reqp->id);
	}
}

static void
uip_retr_done(c, status)
wf_thrd *c;
int status;
{
	/* UIP connection might have dropped, but it's harmless since
	 * session ID is guaranteed to be unique and wf_thrd has "deepcopy"
	 * of necessary member variables.
	 */

	switch (status) {
	case WF_REP_OK:
		req_send(&c->req, "210.%03d Transfer complete.\n", c->req.id);
		break;
	case WF_REP_WAIT:
		req_send(&c->req, "210 %03d Transfer ongoing.\n", c->req.id);
		break;
	default:
		req_send(&c->req, "510.%03d Transfer failed, error=%d\n",
			 c->req.id, status);	/* XXX */
		break;
	}
	if (! (c->flag & WF_THREAD_BUSY)) {
		FREE(c->fhp);
		thrd_free(c);
	}
}

static void
uip_retr_loop(c, status)
wf_thrd *c;
int status;
{
	if (status == WF_REP_OK) {
		status = uip_retr_2(&c->req, c->pathname);
		if (status != WF_REP_WAIT && c->pathname)
			free(c->pathname);
		FREE(c->fhp);
		thrd_free(c);
	} else {
		/* error listing dir */
		if (c->pathname) free(c->pathname);
		uip_retr_done(c, status);
	}
}

static int
uip_retr_2(reqp, pathname)
wf_req *reqp;
char *pathname;
{
	int ret;
	wf_ih ih;
	wf_thrd *c;

	c = thrd_alloc();
	c->req = *reqp;
	c->fhp = ALLOC(wf_fh);

	if (pathname[0] == '+') {
		ret = root_fhn2ih(pathname, &ih);
	} else {
		ret = root_name2ih(pathname, &ih);
	}
	if (ret == WF_REP_OK || ret == WF_REP_WAIT) {
		c->parent_volp = ih.parent_volp;
		c->child_volp = ih.child_volp;
		ih2fh(&ih, c->fhp);
		if (ret == WF_REP_OK) {
			c->reply = uip_retr_done;
		} else {
			c->pathname = pathname;
			c->reply = uip_retr_loop;
		}
		if (ih.filep->fattr.type == NFDIR)
			proto_readdir(c);
		else
			proto_read(c);
	}
	else {
		uip_retr_done(c, ret);
	}
	return ret;
}

static void
uip_retr(reqp, arg)
wf_req *reqp;
char *arg;
{
	req_send(reqp, "220 %03d fil %s\n", reqp->id, arg);
	uip_retr_2(reqp, strdup(arg));
}

static void
uip_load(reqp, arg)
wf_req *reqp;
char *arg;
{
	int ret;
	wf_ih ih;

	ret = root_name2ih(arg, &ih);
	if (ret == WF_REP_OK) {
		req_send(reqp, "220-%03d vol %s\n", reqp->id, arg);
		req_send(reqp, "211.%03d Volume \"%s\" is now visible.\n",
			 reqp->id, arg);
	}
	else {
		req_send(reqp, "520-%03d vol %s\n", reqp->id, arg);
		req_send(reqp, "511.%03d \"%s\": No such volume.\n",
			 reqp->id, arg);
	}
}

static void
uip_unload(reqp, arg)
wf_req *reqp;
char *arg;
{
	int ret;

	ret = root_deletevol(arg);
	if (ret == WF_REP_OK) {
		req_send(reqp, "220-%03d vol %s\n", reqp->id, arg);
		req_send(reqp, "211.%03d Volume \"%s\" is now invisible.\n",
			 reqp->id, arg);
	} else {
		req_send(reqp, "520-%03d vol %s\n", reqp->id, arg);
		req_send(reqp, "511.%03d \"%s\": No such volume.\n",
			 reqp->id, arg);
	}
}

static void
uip_close(reqp)
wf_req *reqp;
{
	int optlen, sotype;

	if (reqp->flag & WF_REQ_PRIV) {
		dlog("privileged connection closed");
	}
	optlen = sizeof(int);
	getsockopt(reqp->so, SOL_SOCKET, SO_TYPE, &sotype, &optlen);
	if (sotype == SOCK_STREAM) {
		so_clear(reqp->so);
		so_unregister(reqp->so, WF_SO_ALL);
		close(reqp->so);
		uip_free(reqp);
	}
	if (cs_ftp_debug == reqp) {	/* XXX */
		cs_ftp_debug = (wf_req *) 0;
	}
}

static
void
uip_parse(reqp, buf)
wf_req *reqp;
char *buf;
{
	int ret;
	char *arg;

	if (! isalpha(buf[0])) {
		req_send(reqp, "510.000 Illegal command.\n");
		return;
	}
	reqp->id = uip_id();
	arg = buf+5;
	switch (buf[0]) {
	case 'D':
		if (strncmp(buf, "DMSG", 4) == 0) {
			uip_dmsg(reqp, arg);
		}
		else if (strncmp(buf, "DIAG", 4) == 0) {
			uip_diag(reqp, arg);
		}
		break;
	case 'E':
		if (strncmp(buf, "EMSG", 4) == 0) {
			uip_emsg(reqp, arg);
		}
		break;
	case 'F':
		if (strncmp(buf, "FILL", 4) == 0) {
		}
		break;
	case 'J':
		if (strncmp(buf, "JUNK", 4) == 0) {
		}
		break;
	case 'R':
		if (strncmp(buf, "RTOC", 4) == 0) {
			uip_rtoc(reqp, arg);
		}
		else if (strncmp(buf, "RETR", 4) == 0) {
			uip_retr(reqp, arg);
		}
		break;
	case 'L':
		if (strncmp(buf, "LOAD", 4) == 0) {
			uip_load(reqp, arg);
		}
		break;
	case 'U':
		if (strncmp(buf, "UNLD", 4) == 0) {
			uip_unload(reqp, arg);
		}
		break;
	case 'Q':
		if (strncmp(buf, "QUIT", 4) == 0) {
			uip_close(reqp);
			break;
		}
	default:
		req_send(reqp, "513.%03d Unknown command.\n", reqp->id);
		break;
	}
}

/* WWUIP over TCP and "connected UDP" */
static void
uip_recv(reqp)
wf_req *reqp;
{
	char *buf, *nl;
	long navail;
	int nread, ret, optlen, sinlen;
	int so = reqp->so;

	/*
	 * check data
	 */
	ioctl(so, FIONREAD, &navail);
	nread = MIN(navail, NFS_MAXDATA);
	/*
	 * Beware: FIONREAD gives incorrect value for UDP sockets.
	 * We cannot trust "nread", but at least it's larger than
	 * actually available bytes. 'mbuf' structure is assumed here,
	 * but even if it doesn't hold, correct UDP implementation
	 * should provide correct "nread" values.
	 */
	if (nread == 0) {
		optlen = sizeof(int);
		getsockopt(so, SOL_SOCKET, SO_ERROR, &ret, &optlen);
		if (ret)
			dlog("uip_recv: %s", sys_errlist[ret]);
		uip_close(reqp);
		return;
	}

	/*
	 * get one line
	 */
	buf = so_getbuf(so);
	sinlen = sizeof(reqp->sin);
	ret = recvfrom(so, buf, nread, MSG_PEEK,
		       (struct sockaddr *) &reqp->sin, &sinlen);
	/*
	 * recvfrom() gives number of bytes read, and it's *correct*.
	 */
	buf[ret] = '\0';
	if ((nl = strchr(buf, '\n')) == NULL) return;
	nread = nl - buf + 1;
	nread = recvfrom(so, buf, nread, 0,
			 (struct sockaddr *) &reqp->sin, &sinlen);
	if (nread == 1)
		return;		/* newline alone doesn't make sense */
	if (buf[nread-2] == '\r') {
		/* telnet oddity here */
		buf[nread-2] = '\0';
	} else {
		buf[nread-1] = '\0';
	}
#ifdef DEBUG_UIP
	dlog("%s.%d -> %s", inet_ntoa(reqp->sin.sin_addr),
	     ntohs(reqp->sin.sin_port), buf);
#endif
	if (inet_netof(reqp->sin.sin_addr) == inet_netof(cs_ipaddr)
	    || reqp->sin.sin_addr.s_addr == inet_addr("127.0.0.1")) {
		/*
		 * Accept request only from the same network.
		 * Further access control can be provided in each volume.
		 */
		uip_parse(reqp, buf);
	} else {
		req_send(reqp, "510.000 Access denied.\n");
		dlog("Invalid access from %s", inet_ntoa(reqp->sin.sin_addr));
	}
}

static void
uip_recvfrom(so)
int so;
{
	static wf_req req;	/* XXX bug if two UIP/UDP come simutaneously */

	bzero(&req, sizeof(req));
	req.so = so;
	uip_recv(&req);
}

/* start WWUIP over TCP */
static void
uip_accept(so)
int so;
{
	int sinlen;
	wf_req *p = uip_alloc();

	/*
	 * accept
	 */
	sinlen = sizeof(sockaddr_in);
	p->so = accept(so, (struct sockaddr *) &p->sin, &sinlen);
	if (p->so < 0) {
		errno_diag();
		return;
	}

	/*
	 * check peer privilege
	 */
	if (bcmp(&p->sin.sin_addr, &cs_ipaddr, sizeof (struct in_addr)) == 0) {
		if (ntohs(p->sin.sin_port) < IPPORT_RESERVED) {
			p->flag |= WF_REQ_PRIV;
			dlog("privileged connection established");
		}
	} else {
#if 0
		/* TCP connection from other hosts.
		 * Reject it to conserve open file descriptor.
		 */
		close(p->so);
		return;
#endif
	}
	so_register(p->so, WF_SO_READ);
	so_callback(p->so, uip_recv, p);
}

/*
 * WWUIP/TCP for telnet,
 * WWUIP/UDP for Tcl/Tk and other utilities.
 *
 * UDP version is preferable since it doesn't consume file descriptor..
 */
void
uip_start()
{
	int opt, tcp_so, udp_so;
	struct sockaddr_in sin;
	int ret;

	/*
	 * create tcp socket
	 */
	tcp_so = socket(AF_INET, SOCK_STREAM, 0);
	if (tcp_so < 0) {
		errno_diag();
		return;
	}
	opt = 1;
#if 0	/* for debug_malloc ? */
	ioctl(tcp_so, FIONBIO, &opt);
#endif
	setsockopt(tcp_so, SOL_SOCKET, SO_REUSEADDR, (char *)&opt, sizeof(int));
	so_register(tcp_so, WF_SO_READ);
	bzero(&sin, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = htonl(INADDR_ANY);
	sin.sin_port = htons(8002);
	ret = bind(tcp_so, (struct sockaddr *) &sin, sizeof (sin));
	if (ret < 0) {
		errno_diag();
		return;
	}
	listen(tcp_so, 1);
	so_callback(tcp_so, uip_accept, tcp_so);

	/*
	 * create udp socket
	 */
	udp_so = socket(AF_INET, SOCK_DGRAM, 0);
	if (udp_so < 0) {
		errno_diag();
		return;
	}
	opt = 1;
	ioctl(udp_so, FIONBIO, &opt);
	so_register(udp_so, WF_SO_READ);
	bzero(&sin, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = htonl(INADDR_ANY);
	sin.sin_port = htons(8002);
	ret = bind(udp_so, (struct sockaddr *) &sin, sizeof (sin));
	if (ret < 0) {
		errno_diag();
		return;
	}
	so_callback(udp_so, uip_recvfrom, udp_so);
}

