/* 
 * WorldWide File System
 * Copyright (c) 1992,1993 Youki Kadobayashi
 * Copyright (c) 1992,1993 Osaka University
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that the following conditions
 * are met:
 * 1. Both the copyright notice and this permission notice appear in
 *    all copies of the software, derivative works or modified versions,
 *    and any portions thereof, and that both notices appear in
 *    supporting documentation.
 * 2. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Osaka University
 *      and its contributors.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE DEVELOPER ``AS IS'' AND OSAKA
 * UNIVERSITY DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
 * WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Osaka University requests users of this software to return to
 *
 *  Youki Kadobayashi
 *  Department of Information and Computer Sciences
 *  Osaka University, Toyonaka 560, Osaka, Japan
 *
 * any improvements or extensions that they make and grant Osaka
 * University the rights to redistribute these changes.
 */
static char *AtFSid = "$Header: util.c[109.2] Sun Dec 19 02:49:00 1993 youki-k@is.aist-nara.ac.jp saved $";

#include <sys/types.h>
#include <sys/file.h>		/* R_OK */
#include <sys/stat.h>		/* for struct stat */
#ifndef R_OK
#include <unistd.h>
#endif
#include <string.h>		/* for strchr */
#ifdef __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif
#include "wfs.h"
#include "global.h"
#include "util.h"

/* queue manipulation functions */
qelem *
q_alloc()
{
	qelem *q;
	q = (qelem *) ALLOC(qelem);
	q->q_forw = q->q_back = q;
	return q;
}

void
q_insert(elem, pred)
qelem *elem, *pred;
{
	qelem *p = pred->q_forw;
	elem->q_back = pred;
	elem->q_forw = p;
	pred->q_forw = elem;
	p->q_back = elem;
}

void
q_remove(elem)
qelem *elem;
{
	qelem *p = elem->q_forw;
	qelem *p2 = elem->q_back;
	p2->q_forw = p;
	p->q_back = p2;
}

int
q_len(elem)
qelem *elem;
{
	qelem *p;
	int len = 0;
	ITER(p, qelem, elem) {
		++len;
	}
	return len;
}

void
q_allfree(head)
qelem *head;
{
	qelem *p, *p2;
	ITER2(p, p2, qelem, head) {
		FREE(p);
	}
	head->q_forw = head;
	head->q_back = head;
}

/* memory allocation functions */
void *
alloc_and_copy(db_alist		v, vlen)
db_dcl
void *v;
int vlen;
{
	void *p;
	p = debug_malloc(db_alist vlen);
	bcopy(v, p, vlen);
	return p;
}

void *
alloc_and_clear(db_alist	vlen)
db_dcl
int vlen;
{
	void *v;
	v = debug_malloc(db_alist vlen);
	bzero(v, vlen);
	return v;
}

void
alloc_start()
{
#ifdef DEBUG_MALLOC
	char path[MAXPATHLEN];
	union dbmalloptarg m;

	sprintf(path, "%s/log/malloc.log", cs_topdir);
	m.str = path;
	dbmallopt(MALLOC_ERRFILE, &m);
#endif
}

/* Dig a deep directory, if needed. Return number of mkdir calls made. */
int
mkdirs(path)
char *path;
{
	/* strings are assumed to be writable. "-fwritable-strings" is
	 * required on some systems (e.g. NeXT).
	 */
	char *p = path, *p2;
	int calls = 0;

	p2 = strrchr(p, '/');
	*p2 = '\0';
	if (access(path, W_OK|X_OK) == 0) {
		*p2 = '/';
		return 0;
	}
	*p2 = '/';
	while(p2 = strchr(p, '/')) {
		*p2 = '\0';
		if (access(path, F_OK) < 0) {	/* D_OK ? */
			mkdir(path, 0770);
			++calls;
		}
		*p2 = '/';
		p = p2+1;
	}
	return calls;
}

int
validate_dir(dir)
char *dir;
{
	char path[MAXPATHLEN];
	struct stat statbuf;

	if (stat(dir, &statbuf) < 0) {
		return -1;
	}
	if (statbuf.st_mode & S_IFDIR) {
		sprintf(path, "%s/usr", dir);
		if (stat(path, &statbuf) == 0) {
			errno = EINVAL;		/* invalid argument */
			return -1;
		}
	} else {
		errno = ENOTDIR;		/* not a directory */
		return -1;
	}
	return 0;
}

void
csd_abort(ret)
int ret;
{
	syslog(LOG_INFO, "Exiting with status %d", ret);
	exit(ret);
}

/* create background process */
int
background()
{
	int pid, retry = 0;

 resume:
	pid = fork();

	/* firewall */
	if (pid < 0 && retry < 5) {
		sleep(1);
		errno_diag();
		++retry;
		goto resume;
	}

	/* child */
	if (pid == 0) {
		cs_pid = getpid();
		foreground = 0;
	}
	return pid;
}

/* acquire privileged port. */
int
bind_resv_port(so, portp)
int so;
unsigned short *portp;
{
	struct sockaddr_in sin;
	unsigned short port;
	int ret;

#if 0
	bzero(&sin, sizeof(sin));
	sin.sin_family = AF_INET;

	port = IPPORT_RESERVED;
	do {
		--port;
		sin.sin_port = htons(port);
		ret = bind(so, (struct sockaddr *) &sin, sizeof (sin));
	} while (ret < 0 && port > IPPORT_RESERVED/2);

	if (ret == 0) {
		*portp = port;
	}
	return ret;
#else
	/* Current version of csd does not require privileged port
	 * since it does not attempt to forward NFS packet, unlike amd.
	 * I guess conventional NFS server requires privileged port
	 * to authenticate request from clients.
	 */
	port = 8000;
	bzero(&sin, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_port = htons(port);
	ret = bind(so, (struct sockaddr *) &sin, sizeof (sin));
	if (ret < 0) {
		errno_diag();
	}
	*portp = port;
	return ret;
#endif
}

/* do the printf to a stream socket. */
#ifdef __STDC__
int
tcp_send(int so, char *fmt, ...)
{
	va_list ap;
	static char buf[MAXPATHLEN + 32];

	va_start(ap, fmt);
	vsprintf(buf, fmt, ap);
	va_end(ap);
	return send(so, buf, strlen(buf), 0);
}
#else

int
tcp_send(va_alist)
va_dcl
{
	va_list ap;
	int so;
	char *fmt;
	static char buf[MAXPATHLEN + 32];

	va_start(ap);
	so = va_arg(ap, int);
	fmt = va_arg(ap, char *);
	vsprintf(buf, fmt, ap);
	va_end(ap);
	return send(so, buf, strlen(buf), 0);
}
#endif

/* do the printf to a stream socket. */
#ifdef __STDC__
int
req_send(wf_req *reqp, char *fmt, ...)
{
	va_list ap;
	static char buf[MAXPATHLEN + 32];

	va_start(ap, fmt);
	vsprintf(buf, fmt, ap);
	va_end(ap);
#ifdef DEBUG_UIP
	dlog("%s.%d <- %s", inet_ntoa(reqp->sin.sin_addr),
	     ntohs(reqp->sin.sin_port), buf);
#endif
	return sendto(reqp->so, buf, strlen(buf), 0,
		      (struct sockaddr *) &reqp->sin, sizeof(reqp->sin));
}
#else

int
req_send(va_alist)
va_dcl
{
	va_list ap;
	wf_req *reqp;
	char *fmt;
	static char buf[MAXPATHLEN + 32];

	va_start(ap);
	reqp = va_arg(ap, wf_req *);
	fmt = va_arg(ap, char *);
	vsprintf(buf, fmt, ap);
	va_end(ap);
#ifdef DEBUG_UIP
	dlog("%s.%d <- %s", inet_ntoa(reqp->sin.sin_addr),
	     ntohs(reqp->sin.sin_port), buf);
#endif
	return sendto(reqp->so, buf, strlen(buf), 0,
		      (struct sockaddr *) &reqp->sin, sizeof(reqp->sin));
}
#endif

/* daily. */

int cs_year;
int cs_month = -1;
int cs_day;
char cs_date[7];
int cs_serial;

void
getdate()
{
	struct tm *p;
	long t = realtime();
#ifdef TRACE
	if (cs_month >= 0)
		trace_report();
#endif
	/*
	 * set daily timer
	 */
	dlog("daily: update time variables");
	/* wake me up tomorrow */
	timeout_set(t - (t % (24 * 3600)) + 24 * 3600, getdate, 0);

	/*
	 * set time variables
	 */
	p = gmtime(&t);
	cs_year = p->tm_year;
	cs_month = p->tm_mon;	/* beware: Jan=0, Dec=11. */
	cs_day = p->tm_mday;
	sprintf(cs_date, "%02d%02d%02d", cs_year, cs_month +1, cs_day);
	cs_serial = 0;
}

/* replacement for system(). */
void
unix_command(cmd)
char *cmd;
{
	char *p, *p2;
	int i, pid, status;
	char *arg[16];

	switch (pid = fork()) {
	case 0:
		/* child process */
		for (i = 0; i < NOFILE; ++i)
			close(i);
		for (p = cmd, i = 0; p2 = strchr(p, ' '); p = p2+1, ++i) {
			*p2 = '\0';
			arg[i] = p;
		}
		arg[i] = p;
		arg[++i] = (char *)0;
		execv(arg[0], arg);
		exit(0);
	case -1:
		errno_diag();
		return;
	default:
		/* parent process */
		dlog(cmd);
		wait(&status);	/* XXX async wait3 version appropriate */
	}
}

long
compute_blocks(size)
long size;
{
	/* fake st_blocks */
	return ((size + 1024) / 1024 + 1);
}

/* Basic part of tm2time came from: Atsushi Onoe <onoe@sm.sony.co.jp>
 * If your libc has mktime() function, you can use it instead.
 */
static int m2yday[] = {
/*
 *	jan feb mar  apr may  jun  jul  aug  sep  oct  nov  dec
 *	31  28  31   30   31   30   31   31   30   31   30   31
 */
	0,  31, 59,  90, 120, 151, 181, 212, 243, 273, 304, 334
};

time_t
tm2time(tm)
struct tm *tm;
{
	int clock;

	clock = (tm->tm_year - 70) * 365 + (tm->tm_year - 72) / 4
		+ m2yday[tm->tm_mon] + tm->tm_mday;
	if ((tm->tm_year - 72) / 4 == 0 && tm->tm_mon < 2)
		--clock;
	clock *= 24;
	clock += tm->tm_hour;
	clock *= 60;
	clock += tm->tm_min;
	clock *= 60;
	clock += tm->tm_sec;
	return ((time_t)clock);
}

#ifndef DEBUG_MALLOC
#if defined(NeXT) || defined(vax) || defined(ultrix) || defined(sony) || defined(luna)
char *
strdup(str)
Const char *str;
{
	char *sp;
	int len;

	len = strlen(str);
	sp = (char *) malloc(len+1);
	bcopy(str, sp, len);
	sp[len] = 0;

	return sp;
}
#endif
#endif
