#!/usr/local/bin/perl
#
$usg = "urlrename: Rename URLs using perl expression.
             Intended as a postprocessing filter for \"htget.pl\".

Usage:	urlrename [-d <dir>] [-pr] <perl expression> <files...>
	-d <dir>	-- directory to save renamed pages
	-p		-- pipe. take filenames from standard input
	-r		-- recursively process files.
	-v		-- verbose.
Example: urlrename -r 's|http://info.cern.ch:80|wwfs:/cern-web|' .
";
# Author: Youki Kadobayashi <youki@wide.ad.jp>
#

require '/etc/csd.pl';
unshift(@INC, "$WWFSDIR/lib");
require 'url.pl';
require 'dirutil.pl';
require 'find.pl';

require 'getopts.pl';
do Getopts('d:prv');
$prefix = $opt_d ? $opt_d : "../url_renamed";
if ($opt_r && $opt_d !~ m|^/|) {
	die "Directory for -d must be absolute path if using -r option.\n";
}

if ($#ARGV < $[) {
	die $usg;
}

$/ = '>';	# input record separator
$expr = shift @ARGV;
if ($opt_r) {
	die $usg if $opt_p;

	foreach $file (@ARGV) {
		&find($file);
	}
} else {
	if ($opt_p) { chop ($_ = <STDIN>); push (@ARGV, $_); }
	foreach $file (@ARGV) {
		($dir, $file) = &basename($file);
		&urlrename($dir, $dir, $file);
		if ($opt_p) { chop ($_ = <STDIN>); push (@ARGV, $_); }
	}
}

sub wanted {
	return 1 unless (-f $_);
	if (/^.*\.html$/) {
		print STDERR "urlrename: $name\n" if $opt_v;
		&urlrename(".", $dir, $_);
	} elsif ($_ eq ".url") {
		open(URL, $_);
		$url = <URL>;
		close(URL);

		# keep new URL...
		eval "\$url =~ $expr";
		&mkdirhier("$prefix/$dir") || die "mkdir: $!\n";
		open(URL, "> $prefix/$dir/.url") || die "new url: $!\n";
		print URL $url;
		close(URL);
	} else {
		print STDERR "mv: $name to $prefix/$name\n" if $opt_v;
		rename($_, "$prefix/$name");
	}
	1;
}

sub urlrename {
	local ($reldir, $dir, $file) = @_;
	local ($here, $header, $url, $trailer,
	       $type,$host,$port,$path,$req,
	       $thistype,$thishost, $thisport,$thispath);
	# finddir goes mad if $_ gets modified
	local ($_);

	open(URL, "$reldir/.url") || die "$file: cannot map to url\n";
	chop ($here = <URL>);
	close(URL);
	($thistype,$thishost, $thisport, $thispath, $thisreq) =
		&url'parse(undef,undef,undef,undef,$here);

	&mkdirhier("$prefix/$dir") || die "mkdir: $!\n";
	open(OUT, ">$prefix/$dir/$file") || die "$prefix/$dir/$file: $!\n";
	select(OUT);
	open(IN, "$reldir/$file") || die "$dir/$file: $!\n";
	while (<IN>) {
		if (/(<a[^>]*href\s*=\s*")([^"]+)("[^>]*>)/i) {
			$header = $` . $1;
			$url = $2;
			$trailer = $3;

			if ($url =~ m|/|) {
				($type,$host,$port,$path,$req) =
					&url'parse($thistype,$thishost,
						$thisport,$thispath,$url);
				$url = "$type:";
				$url .= "//$host" if (defined $host);
				$url .= ":$port" if (defined $port);
				$url .= $req;
				if (eval "\$url =~ $expr" && $req =~ /\/$/) {
					$url .= "urlget_dir.html";
				}
				die $@ if $@;
			}
			print $header, $url, $trailer;
			next;
		}
		if (/(<img[^>]*src\s*=\s*")([^"]+)("[^>]*>)/i) {
			$header = $` . $1;
			$url = $2;
			$trailer = $3;

			if ($url =~ m|/|) {
				($type,$host,$port,$path,$req) =
					&url'parse($thistype,$thishost,
						   $thisport,$thispath,$url);
				$url = "$type:";
				$url .= "//$host" if (defined $host);
				$url .= ":$port" if (defined $port);
				$url .= $req;
				eval "\$url =~ $expr";
				die $@ if $@;
			}
			print $header, $url, $trailer;
			next;
		}
		if (/<ISINDEX>/i) {
			$header = $`;
			# libhtmlw/HTMLformat.c told me...
			print $header, "<ISINDEX ACTION=\"$here\">";
			next;
		}
		print $_;
	}
	close(IN);
	close(OUT);
}

# Local Variables:
# mode: cperl
# cperl-indent-level: 8
# cperl-continued-statement-offset: 8
# End:
