;;; -*-Emacs-Lisp-*-
;;; 
;;; WorldWide File System
;;; Copyright (c) 1992,1993 Youki Kadobayashi
;;; Copyright (c) 1992,1993 Osaka University
;;; All rights reserved.
;;;
;;; Permission to use, copy, modify and distribute this software and its
;;; documentation is hereby granted, provided that the following conditions
;;; are met:
;;; 1. Both the copyright notice and this permission notice appear in
;;;    all copies of the software, derivative works or modified versions,
;;;    and any portions thereof, and that both notices appear in
;;;    supporting documentation.
;;; 2. All advertising materials mentioning features or use of this software
;;;    must display the following acknowledgement:
;;;      This product includes software developed by the Osaka University
;;;      and its contributors.
;;; 3. Neither the name of the University nor the names of its contributors
;;;    may be used to endorse or promote products derived from this software
;;;    without specific prior written permission.
;;;
;;; THIS SOFTWARE IS PROVIDED BY THE DEVELOPER ``AS IS'' AND OSAKA
;;; UNIVERSITY DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY DAMAGES
;;; WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
;;;
;;; Osaka University requests users of this software to return to
;;;
;;;  Youki Kadobayashi
;;;  Department of Information and Computer Sciences
;;;  Osaka University, Toyonaka 560, Osaka, Japan
;;;
;;; any improvements or extensions that they make and grant Osaka
;;; University the rights to redistribute these changes.

;;; $Header: wwfs-access-hook.el[109.1] Tue Dec  7 23:38:35 1993 youki-k@is.aist-nara.ac.jp saved $
;;;
;;; This file requires modification to mule.el that adds 'access-hook'
;;; to 'insert-file-contents'.
;;;
;;; You must add the following lines to ~/.emacs if you want to use this
;;; package.
;;;
;;; (if (boundp 'MULE)
;;;    (require 'wwfs-access-hook))


(defvar wwfs-blocking-read t
  "If set to t, block find-file if the requested file is on WWFS.
If set to nil, remove zero-sized buffer of the same name.")

(defvar wwfs-get-program "/usr/local/ww/bin/wwget"
  "Name of the program to request csd to transfer files and/or directories.")

(defun wwfs-insert-file-contents-hook (filename &optional visit)
  "See also wwfs-blocking-read and insert-file-contents."
;;  (debug)
  (if (file-exists-p
       (concat (file-name-directory filename) ".wwfs_fh"))
      (if wwfs-blocking-read
	  (wwfs-get-file filename)
	(wwfs-remove-junk-buffer filename))))

(setq insert-file-contents-access-hook 'wwfs-insert-file-contents-hook)

(defun wwfs-get-file (pathname &optional switches)
  (let ((file (file-name-nondirectory pathname)))
    (message "Retrieving %s via WWFS..." file)
    (call-process wwfs-get-program nil t nil pathname)
    (message "Retrieving %s via WWFS... done." file)))

(defun wwfs-remove-junk-buffer (file)
  (let ((junk-buffer (get-buffer (file-name-nondirectory file))))
    (set-buffer junk-buffer)
    (save-excursion
      (if (and (zerop (buffer-size))
	       (not (buffer-file-name junk-buffer)))
	  (kill-buffer junk-buffer)))))

(provide 'wwfs-access-hook)
